/*
* Brief:	Define default FRU data in here.
*/

#include "BmcType.h"
#include "com_BmcType.h"
#include "com_IPMI_FRU.h"


/********************************** Flash *************************************/
/*
 * Common Header: 		offset = 0,		len = 8
 * chassis info area	offset = 1,		len = 64
 * Board info area:		offset = 9,		len = 96
 * product info area:	offset = 21, 	len = 88
 * ------------------------------------------------------------------
 * 										total = 256 		
 */
const OemFRUData_T Default_FRUData = {
		//common header
		{
			1,			//Format version
			0,			//no internal use area 
			1,			//chassis info area offset / 8 
			9,			//board info area offset / 8
			21,		//product info area offset / 8
			0,			//MultiRecord area offset / 8
			0x0,		//PAD
			0xe0,		//checksum
		},
		
		//chassis info area
		{
			1,			//version
			FRU_CHASSIS_SIZE/8,		//length = 64/8
			MainServerChassis,
			//Data[58]
			{
					(0xc0 | 0x0a),	
					'0', '0', '0', '0', '-', '0', '0', '0', '1', ' ',   		//Chassis Part Number
					(0xc0 | 0x0a),	
					'0', '0', '0', '0', '-', '0', '0', '0', '1', ' ',    		//Chassis Serial Number
					(0xc0 | 0x0a),	
					' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',    			//Extra info
										
			},
			0xc1,	
			0x0,		//Y
			0x0,		//checksum
		},
		
		//board info area
		{
			1,			//version
			FRU_BOARD_SIZE/8,		//length = 96/8
			English,
			{34, 56, 78},			//Number of minutes from 0:00 hrs 1/1/96.	LSbyte first (little endian)
			//Data[75]
			{
				(0xc0 | 0x0a),	
				'S', 't', 'o', 'r', 'l', 'e', 'a', 'd', ' ', ' ',  		//board manufacturer
				(0xc0 | 0x0d), 
				'O', 'p', 't', 'i', 'c', 'a', 'l', ' ', 'F', 'i', 'b', 'e', 'r',   		//board product name
				(0xc0 | 0x08),
				'0', '0', '0', '1', ' ', ' ', ' ', ' ',	//board product serNum
				(0xc0 | 0x08),
				'0', '0', '0', '1', ' ', ' ', ' ', ' ',	//board product partNum
				(0xc0 | 0x08),
				' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',	//board fru file id
				(0xc0 | 0x08),
				' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',	//Extra info

			},
			{0,0,0,0},	//power on count
			{0,0,0,0},	//Total work time
			{0,0,0,0},	//This work time
			0xc1,	
			0x0,		//Y
			0x0,		//checksum
		},
		
		//product info area
		{
			1,			//version
			FRU_PRODUCT_SIZE/8,		//length	=	88/8
			English,	
			//Data[82]
			{
				(0xc0 | 0x0a),
				'S', 't', 'o', 'r',	'l', 'e', 'a', 'd', ' ', ' ', 	//manufacturer name
				(0xc0 | 0x0e),
				'S', 't', 'o', 'r', 'a', 'g', 'e', ' ', 'S', 'e', 'r', 'v', 'e', 'r',	//product name
				(0xc0 | 0x08),
				'0', '0', '0', '0', '0', '0', '0', '1',	//part number
				(0xc0 | 0x08),
				'1', '.', '0', '0', ' ', ' ', ' ', ' ',	//product version
				(0xc0 | 0x08),
				'0','0','0','0','0','0', '0', '1',	//product serNum
				(0xc0 | 0x08), 	
				' ',' ',' ',' ',' ',' ', ' ', ' ',	//Assert Tag
				(0xc0 | 0x08), 		
				' ',' ',' ',' ',' ',' ',' ',' ',	//FRU File ID
				(0xc0 | 0x08), 		
				' ',' ',' ',' ',' ',' ',' ',' ',	//Extra info
			},
			0xc1,	
			0x0,		//Y
			0x0,		//checksum
		}
};


